@@ -6,12 +6,16 @@ ln -sf /etc/puppet/hiera.yaml /etc/hiera.yaml
 HOMEDIR="$homedir"
 USERNAME=`ls -ld $HOMEDIR | awk {'print $3'}`
 GROUPNAME=`ls -ld $HOMEDIR | awk {'print $4'}`
+THT_DIR="/usr/share/openstack-tripleo-heat-templates"
 
 # WRITE OUT STACKRC
 touch $HOMEDIR/stackrc
 chmod 0600 $HOMEDIR/stackrc
 
 cat > $HOMEDIR/stackrc <<-EOF_CAT
+# Clear any old environment that may conflict.
+for key in \$( set | awk -F= '/^OS_/ {print \$1}' ); do unset "\${key}" ; done
+
 export OS_AUTH_TYPE=password
 export OS_PASSWORD=$admin_password
 export OS_AUTH_URL=$auth_url
@@ -133,6 +137,13 @@ if [ "$(hiera mistral_api_enabled)" = "true" ]; then
       openstack workflow env create $TMP_MISTRAL_ENV
   fi
 
+  # Create the default deployment plan from /usr/share/openstack-tripleo-heat-templates
+  # but only if there is no overcloud container in swift yet.
+  if [ -d "$THT_DIR" ] && ! openstack container list -c Name -f value | grep -qe "^overcloud$"; then
+      echo Create default deployment plan
+      openstack workflow execution create tripleo.plan_management.v1.create_deployment_plan '{"container": "overcloud", "use_default_templates": true}'
+  fi
+
   if [ "$(hiera tripleo_validations_enabled)" = "true" ]; then
       echo Execute copy_ssh_key validations
       openstack workflow execution create tripleo.validations.v1.copy_ssh_key
