@@ -13,15 +13,31 @@
 
 import argparse
 import os
+import re
 import sys
 import traceback
 import yaml
 
-# Only permit the template alias versions
-# The current template version should be the last element
+from copy import copy
+
+# Only permit the template alias versions.
+# The current template version should be the last element.
+# As tripleo-heat-templates is a branched repository, this
+# list should contain only the alias name for the current branch.
+# This allows to avoid merging old templates versions aliases.
 valid_heat_template_versions = [
-  'queens',
+  'rocky',
 ]
+
+# We should have up-to-date the heat_template_version
+# for each template, although there are a few needing
+# an specific version, here we can add this exception
+# for certain templates.
+set_heat_template_versions_in_templates = {
+    # Needed for mixed version pingtest.
+    './ci/pingtests/tenantvm_floatingip.yaml' : 'ocata',
+}
+
 current_heat_template_version = valid_heat_template_versions[-1]
 
 required_params = ['EndpointMap', 'ServiceNetMap', 'DefaultPasswords',
@@ -31,7 +47,8 @@ required_params = ['EndpointMap', 'ServiceNetMap', 'DefaultPasswords',
 # transition to generated environments we have two copies of these files,
 # so they need to be listed twice.  Once the deprecated version can be removed
 # the duplicate entries can be as well.
-envs_containing_endpoint_map = ['tls-endpoints-public-dns.yaml',
+envs_containing_endpoint_map = ['no-tls-endpoints-public-ip.yaml',
+                                'tls-endpoints-public-dns.yaml',
                                 'tls-endpoints-public-ip.yaml',
                                 'tls-everywhere-endpoints-dns.yaml',
                                 'tls-endpoints-public-dns.yaml',
@@ -42,6 +59,7 @@ OPTIONAL_SECTIONS = ['workflow_tasks', 'cellv2_discovery']
 REQUIRED_DOCKER_SECTIONS = ['service_name', 'docker_config', 'puppet_config',
                             'config_settings']
 OPTIONAL_DOCKER_SECTIONS = ['docker_puppet_tasks', 'upgrade_tasks',
+                            'deploy_steps_tasks',
                             'pre_upgrade_rolling_tasks',
                             'fast_forward_upgrade_tasks',
                             'fast_forward_post_upgrade_tasks',
@@ -52,7 +70,8 @@ OPTIONAL_DOCKER_SECTIONS = ['docker_puppet_tasks', 'upgrade_tasks',
                             'logging_source', 'logging_groups',
                             'external_deploy_tasks', 'external_post_deploy_tasks',
                             'docker_config_scripts', 'step_config',
-                            'monitoring_subscription']
+                            'monitoring_subscription',
+                            'external_update_tasks', 'external_upgrade_tasks']
 # ansible tasks cannot be an empty dict or ansible is unhappy
 ANSIBLE_TASKS_SECTIONS = ['upgrade_tasks', 'pre_upgrade_rolling_tasks',
                           'fast_forward_upgrade_tasks',
@@ -87,6 +106,47 @@ PARAMETER_DEFINITION_EXCLUSIONS = {'CephPools': ['description',
                                    'InternalApiAllocationPools': ['default'],
                                    'UpdateIdentifier': ['description'],
                                    'key_name': ['default'],
+                                   'CeilometerAgentCentralLoggingSource': ['default'],
+                                   'CeilometerAgentIpmiLoggingSource': ['default'],
+                                   'CeilometerAgentNotificationLoggingSource': ['default'],
+                                   'CinderApiLoggingSource': ['default'],
+                                   'CinderSchedulerLoggingSource': ['default'],
+                                   'CinderVolumeLoggingSource': ['default'],
+                                   'DesignateApiLoggingSource': ['default'],
+                                   'DesignateCentralLoggingSource': ['default'],
+                                   'DesignateMiniDNSLoggingSource': ['default'],
+                                   'DesignateProducerLoggingSource': ['default'],
+                                   'DesignateSinkLoggingSource': ['default'],
+                                   'DesignateWorkerLoggingSource': ['default'],
+                                   'Ec2ApiLoggingSource': ['default'],
+                                   'GlanceApiLoggingSource': ['default'],
+                                   'GnocchiApiLoggingSource': ['default'],
+                                   'HeatApiCfnLoggingSource': ['default'],
+                                   'HeatApiLoggingSource': ['default'],
+                                   'HeatEngineLoggingSource': ['default'],
+                                   'KeystoneLoggingSource': ['default'],
+                                   'MongoDbLoggingSource': ['default'],
+                                   'NeutronApiLoggingSource': ['default'],
+                                   'NeutronDhcpAgentLoggingSource': ['default'],
+                                   'NeutronL3AgentLoggingSource': ['default'],
+                                   'NeutronMetadataAgentLoggingSource': ['default'],
+                                   'NeutronOpenVswitchAgentLoggingSource': ['default'],
+                                   'NovaApiLoggingSource': ['default'],
+                                   'NovaComputeLoggingSource': ['default'],
+                                   'NovaConductorLoggingSource': ['default'],
+                                   'NovaConsoleauthLoggingSource': ['default'],
+                                   'NovaMetadataLoggingSource': ['default'],
+                                   'NovaPlacementLoggingSource': ['default'],
+                                   'NovaSchedulerLoggingSource': ['default'],
+                                   'NovaVncproxyLoggingSource': ['default'],
+                                   'OctaviaApiLoggingSource': ['default'],
+                                   'OctaviaHealthManagerLoggingSource': ['default'],
+                                   'OctaviaHousekeepingLoggingSource': ['default'],
+                                   'OctaviaWorkerLoggingSource': ['default'],
+                                   'OvnMetadataAgentLoggingSource': ['default'],
+                                   'SaharaApiLoggingSource': ['default'],
+                                   'SaharaEngineLoggingSource': ['default'],
+                                   'TripleoUILoggingSource': ['default'],
                                    # There's one template that defines this
                                    # differently, and I'm not sure if we can
                                    # safely change it.
@@ -114,10 +174,10 @@ PARAMETER_DEFINITION_EXCLUSIONS = {'CephPools': ['description',
                                                                'constraints'],
                                    # NOTE(anil): This is a temporary change and
                                    # will be removed once bug #1767070 properly
-                                   # fixed. OVN supports only VLAN, geneve
-                                   # and flat for NeutronNetworkType. But VLAN
-                                   # tenant networks have a limited support
-                                   # in OVN. Till that is fixed, we restrict
+                                   # fixed. OVN supports only VLAN and geneve
+                                   # for NeutronNetworkType. But VLAN tenant
+                                   # networks have a limited support in OVN.
+                                   # Till that is fixed, we restrict
                                    # NeutronNetworkType to 'geneve'.
                                    'NeutronNetworkType': ['description',
                                                           'default',
@@ -132,7 +192,6 @@ PARAMETER_DEFINITION_EXCLUSIONS = {'CephPools': ['description',
                                    'SSLCertificate': ['description',
                                                       'default',
                                                       'hidden'],
-                                   'HostCpusList': ['default', 'constraints'],
                                    'NodeIndex': ['description'],
                                    'name': ['description', 'default'],
                                    'image': ['description', 'default'],
@@ -154,6 +213,7 @@ PARAMETER_DEFINITION_EXCLUSIONS = {'CephPools': ['description',
 PREFERRED_CAMEL_CASE = {
     'ec2api': 'Ec2Api',
     'haproxy': 'HAProxy',
+    'tripleo-ui': 'TripleoUI',
     'metrics-qdr': 'MetricsQdr'
 }
 
@@ -166,12 +226,29 @@ PREFERRED_CAMEL_CASE = {
 VALIDATE_PUPPET_OVERRIDE = {
   # docker/service/sshd.yaml is a variation of the puppet sshd service
   './docker/services/sshd.yaml': True,
+  # docker/services/messaging/*.yaml provide oslo_messaging services
+  './docker/services/messaging/notify-rabbitmq-shared.yaml': False,
+  './docker/services/messaging/notify-rabbitmq.yaml': False,
+  './docker/services/messaging/rpc-rabbitmq.yaml': False,
+  './docker/services/messaging/rpc-qdrouterd.yaml': False,
+  # docker/services/pacemaker/*-rabbitmq.yaml provide oslo_messaging services
+  './docker/services/pacemaker/notify-rabbitmq.yaml': False,
+  './docker/services/pacemaker/rpc-rabbitmq.yaml': False,
   # qdr aliases rabbitmq service to provide alternative messaging backend
   './puppet/services/qdr.yaml': False,
+  # puppet/services/messaging/*.yaml provide oslo_messaging services
+  './puppet/services/messaging/notify-rabbitmq-shared.yaml': False,
+  './puppet/services/messaging/notify-rabbitmq.yaml': False,
+  './puppet/services/messaging/rpc-rabbitmq.yaml': False,
+  './puppet/services/messaging/rpc-qdrouterd.yaml': False,
+
 }
 VALIDATE_DOCKER_OVERRIDE = {
   # docker/service/sshd.yaml is a variation of the puppet sshd service
   './docker/services/sshd.yaml': False,
+  # docker/services/messaging/notify-rabbitmq-shared.yaml does not
+  # deploy container
+  './docker/services/messaging/notify-rabbitmq-shared.yaml': False,
 }
 DEPLOYMENT_RESOURCE_TYPES = [
     'OS::Heat::SoftwareDeploymentGroup',
@@ -182,13 +259,41 @@ DEPLOYMENT_RESOURCE_TYPES = [
     'OS::Heat::StructuredDeployment',
     'OS::TripleO::SoftwareDeployment'
 ]
+CONFIG_RESOURCE_TYPES = [
+    'OS::Heat::SoftwareConfig',
+    'OS::Heat::StructuredConfig'
+]
 
 VALID_ANSIBLE_UPGRADE_TAGS = [ 'common', 'validation', 'pre-upgrade' ]
+WORKFLOW_TASKS_EXCLUSIONS = [
+    './docker/services/octavia/octavia-deployment-config.yaml',
+    './docker/services/ceph-ansible/ceph-external.yaml',
+    './docker/services/ceph-ansible/ceph-osd.yaml',
+    './docker/services/ceph-ansible/ceph-rbdmirror.yaml',
+    './docker/services/ceph-ansible/ceph-client.yaml',
+    './docker/services/ceph-ansible/ceph-mds.yaml',
+    './docker/services/ceph-ansible/ceph-rgw.yaml',
+    './docker/services/ceph-ansible/ceph-base.yaml',
+    './docker/services/ceph-ansible/ceph-mon.yaml',
+    './docker/services/ceph-ansible/ceph-mgr.yaml',
+    './docker/services/skydive/skydive-base.yaml',
+    './docker/services/skydive/skydive-agent.yaml',
+    './docker/services/skydive/skydive-analyzer.yaml',
+]
+
 
 ANSIBLE_TASKS_YAMLS = [
     './extraconfig/pre_network/boot_param_tasks.yaml'
 ]
 
+HEAT_OUTPUTS_EXCLUSIONS = [
+    './puppet/extraconfig/tls/ca-inject.yaml',
+    './puppet/extraconfig/tls/tls-cert-inject.yaml',
+    './deployed-server/deployed-server.yaml',
+    './extraconfig/tasks/ssh/host_public_key.yaml',
+    './extraconfig/pre_network/host_config_and_reboot.yaml'
+]
+
 def exit_usage():
     print('Usage %s <yaml file or directory>' % sys.argv[0])
     sys.exit(1)
@@ -309,7 +414,6 @@ def validate_ceph_role(ceph_role_filename, ceph_role_tpl):
         for role in ceph_role_tpl:
             if role['name'] == 'CephAll':
                 ceph_role_services = role['ServicesDefault']
-                ceph_role_services.remove('OS::TripleO::Services::CephClient')
                 ceph_role_services.remove('OS::TripleO::Services::CephMds')
                 ceph_role_services.remove('OS::TripleO::Services::CephMgr')
                 ceph_role_services.remove('OS::TripleO::Services::CephMon')
@@ -349,7 +453,7 @@ def validate_controller_no_ceph_role(filename, tpl):
                 return 1
     return 0
 
-def validate_with_compute_role_services(role_filename, role_tpl, exclude_service=()):
+def validate_with_compute_role_services(role_filename, role_tpl, exclude_service):
     cmpt_filename = os.path.join(os.path.dirname(role_filename),
                                  './Compute.yaml')
     cmpt_tpl = yaml.load(open(cmpt_filename).read())
@@ -363,28 +467,9 @@ def validate_with_compute_role_services(role_filename, role_tpl, exclude_service
               'ServicesDefault in roles/Compute.yaml'.format(role_filename,
               ', '.join(missing_services)))
         return 1
-
-    cmpt_us = cmpt_tpl[0].get('update_serial', None)
-    tpl_us = role_tpl[0].get('update_serial', None)
-
-    if 'OS::TripleO::Services::CephOSD' in role_services:
-        if tpl_us not in (None, 1):
-            print('ERROR: update_serial in {0} ({1}) '
-                  'is should be 1 as it includes CephOSD'.format(
-                      role_filename,
-                      tpl_us,
-                      cmpt_us))
-            return 1
-    elif cmpt_us is not None and tpl_us != cmpt_us:
-        print('ERROR: update_serial in {0} ({1}) '
-              'does not match roles/Compute.yaml {2}'.format(
-                  role_filename,
-                  tpl_us,
-                  cmpt_us))
-        return 1
-
     return 0
 
+
 def validate_multiarch_compute_roles(role_filename, role_tpl):
     errors = 0
     roles_dir = os.path.dirname(role_filename)
@@ -619,6 +704,12 @@ def validate_docker_service(filename, tpl):
                 print('ERROR: fast_forward_post_upgrade_tasks validation failed')
                 return 1
 
+        if 'workflow_tasks' in role_data and \
+                filename not in WORKFLOW_TASKS_EXCLUSIONS:
+            print('ERROR: workflow_tasks are no longer supported '
+                  'with config-download in %s.' % filename)
+            return 1
+
     if 'parameters' in tpl:
         for param in required_params:
             if param not in tpl['parameters']:
@@ -690,6 +781,185 @@ def validate_service(filename, tpl):
     return 0
 
 
+def _rsearch_keys(d, pattern, search_keynames=False, enter_lists=False):
+    """ Deep regex search through a dict for k or v matching a pattern
+
+    Returns a list of the matched parent keys. Nested keypaths are
+    represented as lists. Looks for either values (default) or keys mathching
+    the search pattern. A key name may also be joined an integer index, when
+    the matched value belongs to a list and enter_lists is enabled.
+
+    Example:
+
+    >>> example_dict = { 'key1' : [ 'value1', { 'key1': 'value2' } ],
+                         'key2' : 'value2',
+                         'key3' : { 'key3a': 'value3a' },
+                         'key4' : { 'key4a': { 'key4aa': 'value4aa',
+                                               'key4ab': 'value4ab',
+                                               'key4ac': 'value1'},
+                                    'key4b': 'value4b'} }
+    >>>_rsearch_keys(example_dict, 'value1', search_keynames=False,
+                     enter_lists=True)
+    [['key1', 0], ['key4', 'key4a', 'key4ac']]
+    >>> _rsearch_keys(example_dict, 'key4aa', search_keynames=True)
+    [['key4', 'key4a', 'key4aa']]
+    >>> _rsearch_keys(example_dict, 'key1', True, True)
+    [['key1', 1, 'key1']]
+
+    """
+    def _rsearch_keys_nested(d, pattern, search_keynames=False,
+                             enter_lists=False, workset=None, path=None):
+        if path is None:
+            path = []
+        # recursively walk through the dict, optionally entering lists
+        if isinstance(d, dict):
+            for k, v in d.items():
+                path.append(k)
+                if (isinstance(v, dict) or enter_lists and
+                    isinstance(v, list)):
+                    # results are accumulated in the upper scope result var
+                    _rsearch_keys_nested(v, pattern, search_keynames,
+                                         enter_lists, result, path)
+
+                if search_keynames:
+                    target = str(k)
+                else:
+                    target = str(v)
+
+                if re.search(pattern, target):
+                    present = False
+                    for entry in result:
+                        if set(path).issubset(set(entry)):
+                            present = True
+                            break
+                    if not present:
+                        result.append(copy(path))
+
+                path.pop()
+
+        if enter_lists and isinstance(d, list):
+            for ind in range(len(d)):
+                path.append(ind)
+                if (isinstance(d[ind], dict) or
+                    enter_lists and isinstance(d[ind], list)):
+                    _rsearch_keys_nested(d[ind], pattern, search_keynames,
+                                         enter_lists, result, path)
+                if re.search(pattern, str(d[ind])):
+                    present = False
+                    for entry in result:
+                        if set(path).issubset(set(entry)):
+                            present = True
+                            break
+                    if not present:
+                        result.append(copy(path))
+
+                path.pop()
+
+        return result
+
+    result = []
+    return _rsearch_keys_nested(d, pattern, search_keynames, enter_lists)
+
+def _get(d, path):
+    """ Get a value (or None) from a dict by path given as a list
+
+    Integer values represent indexes in lists, string values are for dict keys
+    """
+    if not isinstance(path, list):
+        raise LookupError("The path needs to be a list")
+    for step in path:
+        try:
+            d = d[step]
+        except KeyError:
+            return None
+    return d
+
+def validate_service_hiera_interpol(f, tpl):
+    """  Validate service templates for hiera interpolation rules
+
+    Find all {get_param: [ServiceNetMap, ...]} missing hiera
+    interpolation of IP addresses or network ranges vs
+    the names of the networks, which needs no interpolation
+    """
+    def _getindex(lst, element):
+        try:
+            pos = lst.index(element)
+            return pos
+        except ValueError:
+            return None
+
+    if 'ansible' in f or 'endpoint_map' in f:
+        return 0
+
+    failed = False
+    search_keynames = False
+    enter_lists = True
+    if 'outputs' in tpl and 'role_data' in tpl['outputs']:
+        values_found = _rsearch_keys(tpl['outputs']['role_data'],
+                                     'ServiceNetMap',
+                                     search_keynames, enter_lists)
+        for path in values_found:
+            # Omit if external deploy tasks in the path
+            if 'external_deploy_tasks' in path:
+                continue
+
+            # Omit if not a part of {get_param: [ServiceNetMap ...
+            if not enter_lists and path[-1] != 'get_param':
+                continue
+            if enter_lists and path[-1] != 0 and path[-2] != 'get_param':
+                continue
+
+            path_str = ';'.join(str(x) for x in path)
+            # NOTE(bogdando): Omit foo_network keys looking like a network
+            # name. The only exception is allow anything under
+            # str_replace['params'] ('str_replace;params' in the str notation).
+            # We need to escape because of '$' char may be in templated params.
+            query = re.compile(r'\\;str\\_replace\\;params\\;\S*?net',
+                               re.IGNORECASE)
+            if not query.search(re.escape(path_str)):
+                # Keep parsing, if foo_vip_network, or anything
+                # else that looks like a keystore for an IP address value.
+                query = re.compile(r'(?!ip|cidr|addr|bind|host)([^;]\S)*?net',
+                                   re.IGNORECASE)
+                if query.search(re.escape(path_str)):
+                    continue
+
+            # Omit mappings in tht, like:
+            # [NetXxxMap, <... ,> {get_param: [ServiceNetMap, ...
+            if re.search(r'Map.*get\\_param', re.escape(path_str)):
+                continue
+
+            # For the remaining cases, verify if there is a template
+            # (like str_replace) with the expected format, which is
+            # containing hiera(param_name) interpolation
+            str_replace_pos = _getindex(path, 'str_replace')
+            params_pos = _getindex(path, 'params')
+            if str_replace_pos is None or params_pos is None:
+                print("ERROR: Missed hiera interpolation via str_replace "
+                      "in %s, role_data: %s"
+                      % (f, path))
+                failed = True
+                continue
+
+            # Get the name of the templated param, like NETWORK or $NETWORK
+            param_name = path[params_pos + 1]
+            str_replace = _get(tpl['outputs']['role_data'],
+                               path[:(str_replace_pos + 1)])
+            match_interp = re.search("%%\{hiera\(\S+%s\S+\)\}" %
+                                     re.escape(param_name),
+                                     str_replace['template'])
+            if str_replace['template'] is None or match_interp is None:
+                print("ERROR: Missed %%{hiera('... %s ...')} interpolation "
+                      "in str_replace['template'] "
+                      "in %s, role_data: %s" % (param_name, f, path))
+                failed = True
+                continue
+            # end processing this path and go for the next one
+
+    if failed:
+        return 1
+    else:
+        return 0
 
 def validate_upgrade_tasks_duplicate_whens(filename):
     """Take a heat template and starting at the upgrade_tasks
@@ -705,8 +975,8 @@ def validate_upgrade_tasks_duplicate_whens(filename):
             if '  when:' in line:
                 count += 1
                 if count > 1:
-                    print ("ERROR: found duplicate when statements in %s "
-                           "upgrade_task: %s %s" % (filename, line, duplicate))
+                    print("ERROR: found duplicate when statements in %s "
+                          "upgrade_task: %s %s" % (filename, line, duplicate))
                     return 1
                 duplicate = line
             elif ' -' in line:
@@ -738,6 +1008,7 @@ def validate(filename, param_map):
                                 },
                                 ...
                            ]}
+    Returns a global retval that indicates any failures had been in the check progress.
     """
     if args.quiet < 1:
         print('Validating %s' % filename)
@@ -751,7 +1022,18 @@ def validate(filename, param_map):
         # This validation will be applied to all templates not just for those in the services folder.
         if is_heat_template:
             tpl_template_version = str(tpl['heat_template_version'])
-            if tpl_template_version not in valid_heat_template_versions:
+
+            if filename in set_heat_template_versions_in_templates:
+                if set_heat_template_versions_in_templates[filename] != tpl_template_version:
+                    print('ERROR: heat_template_version in template %s '
+                          'must be set to: %s'
+                        % (
+                            filename,
+                            set_heat_template_versions_in_templates[filename]
+                            )
+                        )
+                    return 1
+            elif tpl_template_version not in valid_heat_template_versions:
                 print('ERROR: heat_template_version in template %s '
                       'is not valid: %s (allowed values %s)'
                     % (
@@ -761,7 +1043,8 @@ def validate(filename, param_map):
                     )
                 )
                 return 1
-            if tpl_template_version != current_heat_template_version:
+            if tpl_template_version != current_heat_template_version \
+                    and args.quiet < 2:
                 print('Warning: heat_template_version in template %s '
                       'is outdated: %s (current %s)'
                     % (
@@ -774,31 +1057,31 @@ def validate(filename, param_map):
         if VALIDATE_PUPPET_OVERRIDE.get(filename, False) or (
                 filename.startswith('./puppet/services/') and
                 VALIDATE_PUPPET_OVERRIDE.get(filename, True)):
-            retval = validate_service(filename, tpl)
+            retval |= validate_service(filename, tpl)
 
+        if re.search(r'(puppet|docker)\/services', filename):
+            retval |= validate_service_hiera_interpol(filename, tpl)
 
         if filename.startswith('./docker/services/logging/'):
-            retval = validate_docker_logging_template(filename, tpl)
+            retval |= validate_docker_logging_template(filename, tpl)
         elif VALIDATE_DOCKER_OVERRIDE.get(filename, False) or (
                 filename.startswith('./docker/services/') and
                 VALIDATE_DOCKER_OVERRIDE.get(filename, True)):
-            retval = validate_docker_service(filename, tpl)
+            retval |= validate_docker_service(filename, tpl)
 
         if filename.endswith('hyperconverged-ceph.yaml'):
-            retval = validate_hci_compute_services_default(filename, tpl)
+            retval |= validate_hci_compute_services_default(filename, tpl)
 
         if filename.startswith('./roles/'):
             retval = validate_role_name(filename)
 
-        if filename.startswith('./roles/ComputeHCI.yaml') or \
-                filename.startswith('./roles/ComputeHCIOvsDpdk.yaml'):
+        if filename.startswith('./roles/ComputeHCI.yaml'):
             retval |= validate_hci_computehci_role(filename, tpl)
 
         if filename.startswith('./roles/ComputeOvsDpdk.yaml') or \
                 filename.startswith('./roles/ComputeSriov.yaml') or \
                 filename.startswith('./roles/ComputeOvsDpdkRT.yaml') or \
-                filename.startswith('./roles/ComputeSriovRT.yaml') or \
-                filename.startswith('./roles/ComputeHCIOvsDpdk.yaml'):
+                filename.startswith('./roles/ComputeSriovRT.yaml'):
             exclude = [
                 'OS::TripleO::Services::OVNController',
                 'OS::TripleO::Services::ComputeNeutronOvsAgent',
@@ -806,36 +1089,42 @@ def validate(filename, param_map):
                 'OS::TripleO::Services::NeutronVppAgent',
                 'OS::TripleO::Services::Vpp',
                 'OS::TripleO::Services::NeutronLinuxbridgeAgent']
-            retval = validate_with_compute_role_services(filename, tpl, exclude)
+            retval |= validate_with_compute_role_services(filename, tpl, exclude)
 
         if filename.startswith('./roles/ComputeRealTime.yaml'):
             exclude = [
                 'OS::TripleO::Services::Tuned',
             ]
-            retval = validate_with_compute_role_services(filename, tpl, exclude)
+            retval |= validate_with_compute_role_services(filename, tpl, exclude)
 
         if filename.startswith('./roles/Hci'):
-            retval = validate_hci_role(filename, tpl)
+            retval |= validate_hci_role(filename, tpl)
 
         if filename.startswith('./roles/Ceph'):
-            retval = validate_ceph_role(filename, tpl)
+            retval |= validate_ceph_role(filename, tpl)
 
         if filename.startswith('./roles/ControllerNoCeph.yaml'):
-            retval = validate_controller_no_ceph_role(filename, tpl)
-
-        if filename in ('./roles/ComputeLocalEphemeral.yaml',
-                        './roles/ComputeRBDEphemeral.yaml'):
-            retval |= validate_with_compute_role_services(filename, tpl)
+            retval |= validate_controller_no_ceph_role(filename, tpl)
 
         if filename == './roles/Compute.yaml':
             retval |= validate_multiarch_compute_roles(filename, tpl)
 
-        if filename.startswith('./network_data_'):
-            retval = validate_network_data_file(filename)
+        # NOTE(hjensas): The routed network data example is very different ...
+        # We need to develop a more advanced validator, probably using a schema
+        # definition instead.
+        # NOTE(mandre): Same goes for the openshift network data where it
+        # contains only a subset of the overcloud networks.
+        if (filename.startswith('./network_data_') and
+                not (filename.endswith('routed.yaml') or
+                     filename.endswith('openshift.yaml'))):
+            result = validate_network_data_file(filename)
+            retval |= result
+        else:
+            result = retval
 
-        if retval == 0 and is_heat_template:
+        if result == 0 and is_heat_template:
             # check for old style nic config files
-            retval = validate_nic_config_file(filename, tpl)
+            retval |= validate_nic_config_file(filename, tpl)
 
     except Exception:
         if filename in ANSIBLE_TASKS_YAMLS:
@@ -859,12 +1148,27 @@ def validate(filename, param_map):
         resources = tpl.get('resources')
         if resources:
             for resource, data in resources.items():
-                if data['type'] not in DEPLOYMENT_RESOURCE_TYPES:
-                    continue
-                if 'name' not in data['properties']:
-                    print('ERROR: resource %s from %s missing name property.'
-                            % (resource, filename))
-                    return 1
+                if data['type'] in DEPLOYMENT_RESOURCE_TYPES:
+                    if 'name' not in data['properties']:
+                        print('ERROR: resource %s from %s missing name property.'
+                                % (resource, filename))
+                        return 1
+
+                elif data['type'] in CONFIG_RESOURCE_TYPES:
+                    if 'outputs' in data['properties']:
+                        if filename in HEAT_OUTPUTS_EXCLUSIONS:
+                            if args.quiet < 1:
+                                print('Resource %s from %s uses Heat '
+                                      'outputs which are not supported with '
+                                      'config-download (ignored due to '
+                                      'exclusions).'
+                                      % (resource, filename))
+                        else:
+                            print('ERROR: resource %s from %s uses Heat '
+                                  'outputs which are not supported with '
+                                  'config-download.'
+                                  % (resource, filename))
+                            return 1
 
     return retval
 
@@ -890,7 +1194,8 @@ def validate_upgrade_tasks(upgrade_tasks):
                         print('ERROR: \'step|int ==\' condition should be evaluated first in when conditions for task (%s)'  % (task))
                         return 1
             else:
-                if (' and ' in whenline) and (' or ' not in whenline):
+                if (' and ' in whenline) and (' or ' not in whenline) \
+                        and args.quiet < 2:
                     print("Warning: Consider specifying \'and\' conditions as a list to improve readability in task: \"%s\"" %  (task_name))
     return 0
 
@@ -899,12 +1204,14 @@ def validate_network_data_file(data_file_path):
         data_file = yaml.load(open(data_file_path).read())
         base_file_path = os.path.dirname(data_file_path) + "/network_data.yaml"
         base_file = yaml.load(open(base_file_path).read())
+        retval = 0
         for n in base_file:
             if n not in data_file:
                 print('ERROR: The following network from network_data.yaml is '
                       'missing or differs in %s : %s'
                       % (data_file_path, n))
-                return 1
+                retval = 1
+        return retval
     except Exception:
         print(traceback.format_exc())
         return 1
@@ -956,9 +1263,11 @@ for base_path in path_args:
             for f in files:
                 file_path = os.path.join(subdir, f)
                 if 'environments/services-docker' in file_path:
-                    print("WARNING: environments/services-docker should not be used "
+                    print("ERROR: environments/services-docker should not be used "
                           "any more, use environments/services instead: %s " %
                           file_path)
+                    failed_files.append(file_path)
+                    exit_val |= 1
 
                 if f.endswith('.yaml') and not f.endswith('.j2.yaml'):
                     failed = validate(file_path, param_map)
